"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var _a = require('semver'), gt = _a.gt, valid = _a.valid, validRange = _a.validRange, satisfies = _a.satisfies;
function diff(sourcePackages, targetPackages) {
    var missingFromTarget = {};
    var needUpgrade = {};
    Object.keys(sourcePackages).forEach(function (name) {
        var version = sourcePackages[name];
        if (version) {
            var targetVersion = targetPackages[name];
            if (targetVersion) {
                if (valid(targetVersion)) {
                    if (valid(version) && gt(version, targetVersion)) {
                        needUpgrade[name] = version;
                    }
                    if (!valid(version) &&
                        validRange(version) &&
                        !satisfies(targetVersion, version)) {
                        needUpgrade[name] = version;
                    }
                }
            }
            else {
                missingFromTarget[name] = version;
            }
        }
    });
    return {
        missingFromTarget: missingFromTarget,
        needUpgrade: needUpgrade
    };
}
function mergeDependencies(sourcePackage, targetPackage, key) {
    var sourceDependencies = sourcePackage[key] || {};
    var targetDependencies = targetPackage[key] || {};
    var _a = diff(sourceDependencies, targetDependencies), missingFromTarget = _a.missingFromTarget, needUpgrade = _a.needUpgrade;
    if (targetPackage[key]) {
        Object.assign(targetPackage[key], missingFromTarget);
        Object.assign(targetPackage[key], needUpgrade);
    }
    else if (sourcePackage[key]) {
        targetPackage[key] = missingFromTarget;
    }
    return targetPackage;
}
function mergePackage(source, target) {
    if (!target) {
        return source;
    }
    var sourcePackage = JSON.parse(source);
    var targetPackage;
    try {
        targetPackage = JSON.parse(target);
    }
    catch (err) {
        throw new Error("package.json is not a valid JSON file - \"" + err.message + "\".\n\n" + target);
    }
    var resultPackage = mergeDependencies(sourcePackage, targetPackage, 'dependencies');
    resultPackage = mergeDependencies(sourcePackage, resultPackage, 'devDependencies');
    return JSON.stringify(resultPackage, null, 2);
}
exports.mergePackage = mergePackage;
//# sourceMappingURL=merge-package.js.map