"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var compiler_1 = require("@angular/compiler");
var compiler_2 = require("@angular/compiler");
var Unparser = /** @class */ (function () {
    function Unparser() {
    }
    Unparser.prototype.unparse = function (ast, interpolationConfig) {
        this._expression = '';
        this._interpolationConfig = interpolationConfig;
        this._visit(ast);
        return this._expression;
    };
    Unparser.prototype.visitPropertyRead = function (ast, context) {
        this._visit(ast.receiver);
        this._expression +=
            ast.receiver instanceof compiler_1.ImplicitReceiver
                ? "" + ast.name
                : "?." + ast.name;
    };
    Unparser.prototype.visitPropertyWrite = function (ast, context) {
        this._visit(ast.receiver);
        this._expression +=
            ast.receiver instanceof compiler_1.ImplicitReceiver
                ? ast.name + " = "
                : "." + ast.name + " = ";
        this._visit(ast.value);
    };
    Unparser.prototype.visitBinary = function (ast, context) {
        this._visit(ast.left);
        this._expression += " " + ast.operation + " ";
        this._visit(ast.right);
    };
    Unparser.prototype.visitChain = function (ast, context) {
        var len = ast.expressions.length;
        for (var i = 0; i < len; i++) {
            this._visit(ast.expressions[i]);
            this._expression += i == len - 1 ? ';' : '; ';
        }
    };
    Unparser.prototype.visitConditional = function (ast, context) {
        this._expression += '(';
        this._visit(ast.condition);
        this._expression += ') ? (';
        this._visit(ast.trueExp);
        this._expression += ') : (';
        this._visit(ast.falseExp);
        this._expression += ')';
    };
    Unparser.prototype.visitPipe = function (ast, context) {
        var _this = this;
        this._visit(ast.exp);
        this._expression += " | " + ast.name;
        ast.args.forEach(function (arg) {
            _this._expression += ':';
            _this._visit(arg);
        });
    };
    Unparser.prototype.visitFunctionCall = function (ast, context) {
        var _this = this;
        this._visit(ast.target);
        this._expression += '(';
        var isFirst = true;
        ast.args.forEach(function (arg) {
            if (!isFirst)
                _this._expression += ', ';
            isFirst = false;
            _this._visit(arg);
        });
        this._expression += ')';
    };
    Unparser.prototype.visitImplicitReceiver = function (ast, context) { };
    Unparser.prototype.visitInterpolation = function (ast, context) {
        for (var i = 0; i < ast.strings.length; i++) {
            this._expression += ast.strings[i];
            if (i < ast.expressions.length) {
                this._expression += this._interpolationConfig.start + " ";
                this._visit(ast.expressions[i]);
                this._expression += " " + this._interpolationConfig.end;
            }
        }
    };
    Unparser.prototype.visitKeyedRead = function (ast, context) {
        this._expression += '(';
        this._visit(ast.obj);
        this._expression += ' || {})';
        this._expression += '[';
        this._visit(ast.key);
        this._expression += ']';
    };
    Unparser.prototype.visitKeyedWrite = function (ast, context) {
        this._visit(ast.obj);
        this._expression += '[';
        this._visit(ast.key);
        this._expression += '] = ';
        this._visit(ast.value);
    };
    Unparser.prototype.visitLiteralArray = function (ast, context) {
        var _this = this;
        this._expression += '[';
        var isFirst = true;
        ast.expressions.forEach(function (expression) {
            if (!isFirst)
                _this._expression += ', ';
            isFirst = false;
            _this._visit(expression);
        });
        this._expression += ']';
    };
    Unparser.prototype.visitLiteralMap = function (ast, context) {
        this._expression += '{';
        var isFirst = true;
        for (var i = 0; i < ast.keys.length; i++) {
            if (!isFirst)
                this._expression += ', ';
            isFirst = false;
            this._expression += ast.keys[i] + ": ";
            this._visit(ast.values[i]);
        }
        this._expression += '}';
    };
    Unparser.prototype.visitLiteralPrimitive = function (ast, context) {
        if (typeof ast.value === 'string') {
            this._expression += "'" + ast.value.replace(Unparser._quoteRegExp, "'") + "'";
        }
        else {
            this._expression += "" + ast.value;
        }
    };
    Unparser.prototype.visitMethodCall = function (ast, context) {
        var _this = this;
        this._visit(ast.receiver);
        this._expression +=
            ast.receiver instanceof compiler_1.ImplicitReceiver
                ? ast.name + "("
                : "?." + ast.name + "(";
        var isFirst = true;
        ast.args.forEach(function (arg) {
            if (!isFirst)
                _this._expression += ', ';
            isFirst = false;
            _this._visit(arg);
        });
        this._expression += ')';
    };
    Unparser.prototype.visitPrefixNot = function (ast, context) {
        this._expression += '!';
        this._visit(ast.expression);
    };
    Unparser.prototype.visitNonNullAssert = function (ast, context) {
        this._visit(ast.expression);
        this._expression += '!';
    };
    Unparser.prototype.visitSafePropertyRead = function (ast, context) {
        this._visit(ast.receiver);
        this._expression += "?." + ast.name;
    };
    Unparser.prototype.visitSafeMethodCall = function (ast, context) {
        var _this = this;
        this._visit(ast.receiver);
        this._expression += "?." + ast.name + "(";
        var isFirst = true;
        ast.args.forEach(function (arg) {
            if (!isFirst)
                _this._expression += ', ';
            isFirst = false;
            _this._visit(arg);
        });
        this._expression += ')';
    };
    Unparser.prototype.visitQuote = function (ast, context) {
        this._expression += ast.prefix + ":" + ast.uninterpretedExpression;
    };
    Unparser.prototype._visit = function (ast) {
        ast.visit(this);
    };
    Unparser._quoteRegExp = /'/g;
    return Unparser;
}());
var sharedUnparser = new Unparser();
function default_1(value) {
    var ast = new compiler_1.Parser(new compiler_1.Lexer()).parseBinding(value, null);
    return sharedUnparser.unparse(ast, compiler_2.DEFAULT_INTERPOLATION_CONFIG);
}
exports.default = default_1;
//# sourceMappingURL=safe-expression.js.map