"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
function securityDataSource() {
    return {
        swagger: '2.0',
        info: {
            title: 'Security',
            version: '1.0.0'
        },
        produces: [],
        host: null,
        schemes: ['http'],
        basePath: null,
        definitions: {
            Error: {
                type: 'object',
                properties: {
                    error: {
                        type: 'object',
                        properties: {
                            message: {
                                type: 'string'
                            }
                        }
                    }
                }
            },
            Role: {
                type: 'object',
                properties: {
                    Id: {
                        type: 'string',
                        'x-key': true
                    },
                    Name: {
                        type: 'string'
                    }
                },
                required: ['Name']
            },
            User: {
                type: 'object',
                properties: {
                    Id: {
                        type: 'string',
                        'x-key': true
                    },
                    UserName: {
                        type: 'string'
                    },
                    Email: {
                        type: 'string'
                    }
                },
                required: ['UserName', 'Email']
            }
        },
        paths: {
            '/auth/change-password': {
                post: {
                    operationId: 'changePassword',
                    parameters: [
                        {
                            in: 'formData',
                            name: 'OldPassword',
                            type: 'string',
                            required: true
                        },
                        {
                            in: 'formData',
                            name: 'NewPassword',
                            type: 'string',
                            required: true
                        }
                    ],
                    responses: {
                        204: {
                            description: ''
                        },
                        default: {
                            description: '',
                            schema: {
                                $ref: '#/definitions/Error'
                            }
                        }
                    }
                }
            },
            '/auth/login': {
                post: {
                    operationId: 'login',
                    parameters: [
                        {
                            in: 'formData',
                            name: 'username',
                            type: 'string',
                            required: true
                        },
                        {
                            in: 'formData',
                            name: 'password',
                            type: 'string',
                            required: true
                        }
                    ],
                    responses: {
                        default: {
                            description: '',
                            schema: {
                                $ref: '#/definitions/Error'
                            }
                        }
                    }
                }
            },
            '/auth/reset-password': {
                post: {
                    operationId: 'resetPassword',
                    parameters: [
                        {
                            in: 'formData',
                            name: 'username',
                            type: 'string',
                            required: true
                        }
                    ],
                    responses: {
                        default: {
                            description: '',
                            schema: {
                                $ref: '#/definitions/Error'
                            }
                        }
                    }
                }
            },
            '/auth/register': {
                post: {
                    operationId: 'registerUser',
                    parameters: [
                        {
                            in: 'body',
                            name: 'User',
                            type: 'object',
                            required: true
                        }
                    ],
                    responses: {
                        default: {
                            description: '',
                            schema: {
                                $ref: '#/definitions/Error'
                            }
                        }
                    }
                }
            },
            '/auth/ApplicationUsers': {
                get: {
                    operationId: 'getUsers',
                    parameters: [
                        {
                            in: 'query',
                            name: '$filter',
                            type: 'string'
                        },
                        {
                            in: 'query',
                            name: '$top',
                            type: 'integer'
                        },
                        {
                            in: 'query',
                            name: '$skip',
                            type: 'integer'
                        },
                        {
                            in: 'query',
                            name: '$orderby',
                            type: 'string'
                        },
                        {
                            in: 'query',
                            name: '$count',
                            type: 'boolean'
                        },
                        {
                            in: 'query',
                            name: '$expand',
                            type: 'string'
                        }
                    ],
                    responses: {
                        '200': {
                            description: '',
                            schema: {
                                properties: {
                                    value: {
                                        items: {
                                            $ref: '#/definitions/User'
                                        },
                                        type: 'array'
                                    }
                                },
                                type: 'object'
                            }
                        },
                        default: {
                            description: '',
                            schema: {
                                $ref: '#/definitions/Error'
                            }
                        }
                    }
                },
                post: {
                    operationId: 'createUser',
                    parameters: [
                        {
                            in: 'body',
                            name: 'User',
                            required: true,
                            schema: {
                                $ref: '#/definitions/User'
                            }
                        }
                    ],
                    responses: {
                        '201': {
                            description: '',
                            schema: {
                                $ref: '#/definitions/User'
                            }
                        },
                        default: {
                            description: '',
                            schema: {
                                $ref: '#/definitions/Error'
                            }
                        }
                    }
                }
            },
            "/auth/ApplicationUsers('{Id}')": {
                delete: {
                    operationId: 'deleteUser',
                    parameters: [
                        {
                            in: 'path',
                            name: 'id',
                            required: true,
                            type: 'string'
                        }
                    ],
                    responses: {
                        '204': {
                            description: ''
                        },
                        default: {
                            description: '',
                            schema: {
                                $ref: '#/definitions/Error'
                            }
                        }
                    }
                },
                get: {
                    operationId: 'getUserById',
                    parameters: [
                        {
                            in: 'path',
                            name: 'Id',
                            required: true,
                            type: 'string'
                        }
                    ],
                    responses: {
                        '200': {
                            description: '',
                            schema: {
                                $ref: '#/definitions/User'
                            }
                        },
                        default: {
                            description: '',
                            schema: {
                                $ref: '#/definitions/Error'
                            }
                        }
                    }
                },
                patch: {
                    operationId: 'updateUser',
                    parameters: [
                        {
                            in: 'path',
                            name: 'Id',
                            required: true,
                            type: 'string'
                        },
                        {
                            in: 'body',
                            name: 'User',
                            required: true,
                            schema: {
                                $ref: '#/definitions/User'
                            }
                        }
                    ],
                    responses: {
                        '200': {
                            description: '',
                            schema: {
                                $ref: '#/User'
                            }
                        },
                        '204': {
                            description: ''
                        },
                        default: {
                            description: '',
                            schema: {
                                $ref: '#/definitions/Error'
                            }
                        }
                    }
                }
            },
            "/auth/ApplicationRoles('{id}')": {
                delete: {
                    operationId: 'deleteRole',
                    parameters: [
                        {
                            in: 'path',
                            name: 'id',
                            required: true,
                            type: 'string'
                        }
                    ],
                    responses: {
                        '204': {
                            description: ''
                        },
                        default: {
                            description: '',
                            schema: {
                                $ref: '#/definitions/Error'
                            }
                        }
                    }
                },
                get: {
                    operationId: 'getRoleById',
                    parameters: [
                        {
                            in: 'path',
                            name: 'id',
                            required: true,
                            type: 'string'
                        }
                    ],
                    responses: {
                        '200': {
                            description: '',
                            schema: {
                                $ref: '#/definitions/Role'
                            }
                        },
                        default: {
                            description: '',
                            schema: {
                                $ref: '#/definitions/Error'
                            }
                        }
                    }
                },
                patch: {
                    operationId: 'updateRole',
                    parameters: [
                        {
                            in: 'path',
                            name: 'id',
                            required: true,
                            type: 'string'
                        },
                        {
                            in: 'body',
                            name: 'role',
                            required: true,
                            schema: {
                                $ref: '#/definitions/Role'
                            }
                        }
                    ],
                    responses: {
                        '200': {
                            description: '',
                            schema: {
                                $ref: '#/definitions/Microsoft.AspNetCore.Identity.EntityFrameworkCore.IdentityRole'
                            }
                        },
                        '204': {
                            description: ''
                        },
                        default: {
                            description: '',
                            schema: {
                                $ref: '#/definitions/Error'
                            }
                        }
                    }
                }
            },
            '/auth/ApplicationRoles': {
                get: {
                    operationId: 'getRoles',
                    parameters: [
                        {
                            in: 'query',
                            name: '$filter',
                            type: 'string'
                        },
                        {
                            in: 'query',
                            name: '$top',
                            type: 'integer'
                        },
                        {
                            in: 'query',
                            name: '$skip',
                            type: 'integer'
                        },
                        {
                            in: 'query',
                            name: '$orderby',
                            type: 'string'
                        },
                        {
                            in: 'query',
                            name: '$count',
                            type: 'boolean'
                        },
                        {
                            in: 'query',
                            name: '$expand',
                            type: 'string'
                        }
                    ],
                    responses: {
                        '200': {
                            description: '',
                            schema: {
                                properties: {
                                    value: {
                                        items: {
                                            $ref: '#/definitions/Role'
                                        },
                                        type: 'array'
                                    }
                                },
                                type: 'object'
                            }
                        },
                        default: {
                            description: '',
                            schema: {
                                $ref: '#/definitions/Error'
                            }
                        }
                    }
                },
                post: {
                    operationId: 'createRole',
                    parameters: [
                        {
                            in: 'body',
                            name: 'Role',
                            required: true,
                            schema: {
                                $ref: '#/definitions/Role'
                            }
                        }
                    ],
                    responses: {
                        '201': {
                            description: '',
                            schema: {
                                $ref: '#/definitions/Role'
                            }
                        },
                        default: {
                            description: '',
                            schema: {
                                $ref: '#/definitions/Error'
                            }
                        }
                    }
                }
            }
        }
    };
}
exports.securityDataSource = securityDataSource;
//# sourceMappingURL=securityDataSource.js.map