"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
var core_1 = require("@angular/core");
var common_1 = require("@angular/common");
var get = require("lodash/get");
var map = require("lodash/map");
var groupBy = require("lodash/groupBy");
var ngx_charts_1 = require("@swimlane/ngx-charts");
var d3_shape_1 = require("d3-shape");
var shared_1 = require("./shared");
var utc_date_pipe_1 = require("./utc-date.pipe");
var ChartComponent = /** @class */ (function () {
    function ChartComponent() {
        this.style = {};
        this.barPadding = 8;
        this.roundDomains = false;
        this.showGridLines = true;
        this.scheme = {
            domain: [
                '#479cc8',
                '#68D5C8',
                '#ff6d41',
                '#cb6992',
                '#e6c54f',
                '#f9777f',
                '#5dbf74',
                '#4db9f2',
                '#FF6D41'
            ]
        };
        this.select = new core_1.EventEmitter();
        this.results = [];
        this.view = [];
        this.curve = d3_shape_1.curveMonotoneX;
        this.afterViewInit = false;
    }
    Object.defineProperty(ChartComponent.prototype, "colors", {
        get: function () {
            return this.scheme.domain;
        },
        set: function (value) {
            if (value) {
                this.scheme.domain = value;
            }
        },
        enumerable: true,
        configurable: true
    });
    ChartComponent.prototype.formatLabel = function (label) {
        if (label instanceof Date) {
            label = new Date(label.getUTCFullYear(), label.getUTCMonth(), label.getUTCDate(), label.getUTCHours(), label.getUTCMinutes(), label.getUTCSeconds(), label.getUTCMilliseconds());
            label = label.toLocaleDateString();
        }
        else {
            label = label.toLocaleString();
        }
        return label;
    };
    ChartComponent.prototype.onSelect = function (event) {
        if (typeof event != 'string') {
            this.select.emit({
                category: event.name,
                value: event.value,
                series: event.series
            });
        }
    };
    ChartComponent.prototype.update = function () {
        var _this = this;
        if (this.data && this.categoryProperty && this.valueProperty) {
            var series = void 0;
            if (this.series) {
                var groups = groupBy(this.data, this.series);
                series = map(groups, function (data, name) {
                    var series = _this.map(data);
                    return {
                        name: name,
                        series: series
                    };
                });
            }
            else {
                series = this.map(this.data);
            }
            if ((this.chartType == 'Line' || this.chartType == 'Area') &&
                !this.series) {
                this.results = [
                    {
                        name: this.categoryProperty,
                        series: series
                    }
                ];
            }
            else {
                this.results = series;
            }
        }
    };
    ChartComponent.prototype.map = function (data) {
        var _this = this;
        return data.map(function (item) {
            var value = get(item, _this.valueProperty);
            var name = get(item, _this.categoryProperty);
            var date = utc_date_pipe_1.parseDate(name);
            if (date != null) {
                name = date;
            }
            return {
                name: name,
                value: +value
            };
        });
    };
    ChartComponent.prototype.ngAfterViewInit = function () {
        this.afterViewInit = true;
    };
    ChartComponent.prototype.ngOnChanges = function () {
        this.update();
    };
    ChartComponent.prototype.ngDoCheck = function () {
        if (this.afterViewInit) {
            var _a = this.view, width = _a[0], height = _a[1];
            var _b = this.container.nativeElement, clientWidth = _b.clientWidth, clientHeight = _b.clientHeight;
            if (width != clientWidth || height != clientHeight) {
                this.view = [clientWidth, clientHeight];
            }
        }
    };
    __decorate([
        core_1.Input(),
        __metadata("design:type", Object)
    ], ChartComponent.prototype, "style", void 0);
    __decorate([
        core_1.Input(),
        __metadata("design:type", String)
    ], ChartComponent.prototype, "title", void 0);
    __decorate([
        core_1.Input(),
        __metadata("design:type", String)
    ], ChartComponent.prototype, "icon", void 0);
    __decorate([
        core_1.Input(),
        __metadata("design:type", String)
    ], ChartComponent.prototype, "titleBackground", void 0);
    __decorate([
        core_1.Input(),
        __metadata("design:type", String)
    ], ChartComponent.prototype, "titleColor", void 0);
    __decorate([
        core_1.Input(),
        __metadata("design:type", String)
    ], ChartComponent.prototype, "series", void 0);
    __decorate([
        core_1.Input(),
        __metadata("design:type", Boolean)
    ], ChartComponent.prototype, "legend", void 0);
    __decorate([
        core_1.Input(),
        __metadata("design:type", String)
    ], ChartComponent.prototype, "chartType", void 0);
    __decorate([
        core_1.Input(),
        __metadata("design:type", Array)
    ], ChartComponent.prototype, "data", void 0);
    __decorate([
        core_1.Input(),
        __metadata("design:type", String)
    ], ChartComponent.prototype, "categoryProperty", void 0);
    __decorate([
        core_1.Input(),
        __metadata("design:type", String)
    ], ChartComponent.prototype, "valueProperty", void 0);
    __decorate([
        core_1.Input(),
        __metadata("design:type", Object)
    ], ChartComponent.prototype, "barPadding", void 0);
    __decorate([
        core_1.Input(),
        __metadata("design:type", Object)
    ], ChartComponent.prototype, "roundDomains", void 0);
    __decorate([
        core_1.Input(),
        __metadata("design:type", Object)
    ], ChartComponent.prototype, "showGridLines", void 0);
    __decorate([
        core_1.Input(),
        __metadata("design:type", Array),
        __metadata("design:paramtypes", [Array])
    ], ChartComponent.prototype, "colors", null);
    __decorate([
        core_1.Output(),
        __metadata("design:type", Object)
    ], ChartComponent.prototype, "select", void 0);
    __decorate([
        core_1.ViewChild('container'),
        __metadata("design:type", core_1.ElementRef)
    ], ChartComponent.prototype, "container", void 0);
    ChartComponent = __decorate([
        core_1.Component({
            selector: 'rz-chart',
            template: "\n    <div #container class=\"chart\" [ngStyle]=\"style\">\n      <ngx-charts-bar-vertical-2d\n        *ngIf=\"chartType == 'Column' && series\"\n        [view]=\"view\"\n        (select)=\"onSelect($event)\"\n        [barPadding]=\"barPadding\"\n        [xAxis]=\"true\"\n        [yAxis]=\"true\"\n        [animations]=\"false\"\n        [scheme]=\"scheme\"\n        [results]=\"results\"\n        [legend]=\"legend\"\n        [xAxisTickFormatting]=\"formatLabel\"\n        [showGridLines]=\"showGridLines\"\n      ></ngx-charts-bar-vertical-2d>\n      <ngx-charts-bar-vertical\n        *ngIf=\"chartType == 'Column' && !series\"\n        [view]=\"view\"\n        (select)=\"onSelect($event)\"\n        [barPadding]=\"barPadding\"\n        [xAxis]=\"true\"\n        [yAxis]=\"true\"\n        [animations]=\"false\"\n        [scheme]=\"scheme\"\n        [results]=\"results\"\n        [legend]=\"legend\"\n        [xAxisTickFormatting]=\"formatLabel\"\n        [showGridLines]=\"showGridLines\"\n      ></ngx-charts-bar-vertical>\n      <ngx-charts-bar-horizontal-2d\n        *ngIf=\"chartType == 'Bar' && series\"\n        [view]=\"view\"\n        (select)=\"onSelect($event)\"\n        [barPadding]=\"barPadding\"\n        [xAxis]=\"true\"\n        [yAxis]=\"true\"\n        [results]=\"results\"\n        [animations]=\"false\"\n        [scheme]=\"scheme\"\n        [legend]=\"legend\"\n        [xAxisTickFormatting]=\"formatLabel\"\n        [showGridLines]=\"showGridLines\"\n      ></ngx-charts-bar-horizontal-2d>\n      <ngx-charts-bar-horizontal\n        *ngIf=\"chartType == 'Bar' && !series\"\n        [view]=\"view\"\n        (select)=\"onSelect($event)\"\n        [barPadding]=\"barPadding\"\n        [xAxis]=\"true\"\n        [yAxis]=\"true\"\n        [animations]=\"false\"\n        [scheme]=\"scheme\"\n        [results]=\"results\"\n        [legend]=\"legend\"\n        [xAxisTickFormatting]=\"formatLabel\"\n        [showGridLines]=\"showGridLines\"\n      ></ngx-charts-bar-horizontal>\n      <ngx-charts-line-chart\n        *ngIf=\"chartType == 'Line'\"\n        [view]=\"view\"\n        (select)=\"onSelect($event)\"\n        [xAxis]=\"true\"\n        [yAxis]=\"true\"\n        [curve]=\"curve\"\n        [animations]=\"false\"\n        [scheme]=\"scheme\"\n        [results]=\"results\"\n        [roundDomains]=\"roundDomains\"\n        [legend]=\"legend\"\n        [showGridLines]=\"showGridLines\"\n      ></ngx-charts-line-chart>\n      <ngx-charts-area-chart\n        *ngIf=\"chartType == 'Area'\"\n        (select)=\"onSelect($event)\"\n        [xAxis]=\"true\"\n        [yAxis]=\"true\"\n        [curve]=\"curve\"\n        [animations]=\"false\"\n        [scheme]=\"scheme\"\n        [view]=\"view\"\n        [results]=\"results\"\n        [roundDomains]=\"roundDomains\"\n        [showGridLines]=\"showGridLines\"\n        [legend]=\"legend\"\n      ></ngx-charts-area-chart>\n    </div>\n  "
        })
    ], ChartComponent);
    return ChartComponent;
}());
exports.ChartComponent = ChartComponent;
var ChartModule = /** @class */ (function () {
    function ChartModule() {
    }
    ChartModule = __decorate([
        core_1.NgModule({
            imports: [common_1.CommonModule, ngx_charts_1.NgxChartsModule, shared_1.SharedModule],
            declarations: [ChartComponent],
            exports: [ChartComponent]
        })
    ], ChartModule);
    return ChartModule;
}());
exports.ChartModule = ChartModule;
