"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var moment = require("moment");
var utc_date_pipe_1 = require("./utc-date.pipe");
function onUserInput(event) {
    // IE 11 Workaround for input placeholder : https://github.com/primefaces/primeng/issues/2026
    if (!this.isKeydown) {
        return;
    }
    this.isKeydown = false;
    var val = event.target.value;
    try {
        var value = this.parseValueFromString(val);
        if (this.isValidSelection(value)) {
            this.updateModel(value);
            this.updateUI();
        }
    }
    catch (err) {
        //invalid date
        this.updateModel(null);
    }
    this.filled = val != null && val.length;
    this.onInput.emit(event);
}
exports.onUserInput = onUserInput;
function defaultLocaleSettings() {
    return {
        firstDayOfWeek: 0,
        dayNames: [
            'Sunday',
            'Monday',
            'Tuesday',
            'Wednesday',
            'Thursday',
            'Friday',
            'Saturday'
        ],
        dayNamesShort: ['Sun', 'Mon', 'Tue', 'Wed', 'Thu', 'Fri', 'Sat'],
        dayNamesMin: ['Su', 'Mo', 'Tu', 'We', 'Th', 'Fr', 'Sa'],
        monthNames: [
            'January',
            'February',
            'March',
            'April',
            'May',
            'June',
            'July',
            'August',
            'September',
            'October',
            'November',
            'December'
        ],
        monthNamesShort: [
            'Jan',
            'Feb',
            'Mar',
            'Apr',
            'May',
            'Jun',
            'Jul',
            'Aug',
            'Sep',
            'Oct',
            'Nov',
            'Dec'
        ],
        today: 'Today',
        clear: 'Clear'
    };
}
exports.defaultLocaleSettings = defaultLocaleSettings;
function toLocaleSettings(locale) {
    var result = defaultLocaleSettings();
    var localeData = moment.localeData(locale);
    result.firstDayOfWeek = localeData.firstDayOfWeek();
    result.dayNames = localeData.weekdays();
    result.dayNamesMin = localeData.weekdaysMin();
    result.dayNamesShort = localeData.weekdaysShort();
    result.monthNames = localeData.months();
    result.monthNamesShort = localeData.monthsShort();
    return result;
}
exports.toLocaleSettings = toLocaleSettings;
function toUTC(value) {
    if (value == null) {
        return value;
    }
    return new Date(Date.UTC(value.getFullYear(), value.getMonth(), value.getDate(), value.getHours(), value.getMinutes(), value.getSeconds(), value.getMilliseconds()));
}
exports.toUTC = toUTC;
function parse(value, utc) {
    if (value != null) {
        var utcCheck = true;
        if (typeof value == 'string') {
            utcCheck = utc_date_pipe_1.ISO8601_DATE_REGEX.test(value);
        }
        value = new Date(value);
        if (utc && utcCheck) {
            value = new Date(value.getUTCFullYear(), value.getUTCMonth(), value.getUTCDate(), value.getUTCHours(), value.getUTCMinutes(), value.getUTCSeconds(), value.getUTCMilliseconds());
        }
        else {
            value = new Date(Date.parse(value));
        }
    }
    return value;
}
exports.parse = parse;
