"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
var core_1 = require("@angular/core");
var common_1 = require("@angular/common");
var get = require("lodash/get");
var set = require("lodash/set");
var core_2 = require("@fullcalendar/core");
var daygrid_1 = require("@fullcalendar/daygrid");
var timegrid_1 = require("@fullcalendar/timegrid");
var interaction_1 = require("@fullcalendar/interaction");
var moment_1 = require("@fullcalendar/moment");
var allLocales = require("@fullcalendar/core/locales-all");
var SchedulerComponent = /** @class */ (function () {
    function SchedulerComponent() {
        this.editable = true;
        this.weekends = true;
        this.defaultView = 'month';
        this.locale = 'en';
        this.firstDayOfWeek = 0;
        this.timeFormat = null;
        this.eventSelect = new core_1.EventEmitter();
        this.eventUpdate = new core_1.EventEmitter();
        this.eventAdd = new core_1.EventEmitter();
        this.eventInit = new core_1.EventEmitter();
        this.loadData = new core_1.EventEmitter();
        this.events = [];
    }
    Object.defineProperty(SchedulerComponent.prototype, "style", {
        get: function () {
            return this.innerStyle;
        },
        set: function (style) {
            this.innerStyle = style;
            if (style && style.hasOwnProperty('height')) {
                var height = style.height;
                if (/px/.test(height)) {
                    this.height = parseInt(height);
                }
            }
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SchedulerComponent.prototype, "data", {
        set: function (value) {
            var _this = this;
            if (!Array.isArray(value)) {
                return;
            }
            this.events = value
                .map(function (data) {
                return {
                    data: data,
                    id: _this.idProperty ? get(data, _this.idProperty) : '',
                    title: _this.titleProperty ? get(data, _this.titleProperty) : '',
                    start: _this.startProperty ? get(data, _this.startProperty) : '',
                    end: _this.endProperty ? get(data, _this.endProperty) : ''
                };
            })
                .filter(function (event) { return event.start != null; })
                .map(function (event) {
                _this.eventInit.next(event);
                return event;
            });
            if (this.calendar) {
                this.calendar.removeAllEventSources();
                this.calendar.addEventSource(this.events);
            }
        },
        enumerable: true,
        configurable: true
    });
    SchedulerComponent.prototype.onViewRender = function (e) {
        var _this = this;
        setTimeout(function () {
            _this.loadData.next({
                start: moment_1.toMoment(e.view.activeStart, _this.calendar).format(),
                end: moment_1.toMoment(e.view.activeEnd, _this.calendar).format()
            });
        });
    };
    SchedulerComponent.prototype.onEventDrop = function (e) {
        this.update(e);
    };
    SchedulerComponent.prototype.onEventResize = function (e) {
        this.update(e);
    };
    SchedulerComponent.prototype.onDayClick = function (e) {
        var date = moment_1.toMoment(e.date, this.calendar);
        var start = date.format();
        var end = start;
        if (e.view.type == 'dayGridMonth') {
            end = date.add(1, 'days').format();
        }
        else if (e.view.type == 'timeGridWeek' || e.view.type == 'timeGridDay') {
            end = date.add(30, 'minutes').format();
        }
        this.eventAdd.emit({ start: start, end: end });
    };
    SchedulerComponent.prototype.ngOnChanges = function (changes) {
        var _this = this;
        if (this.calendar) {
            var defaultDate_1 = changes.defaultDate;
            var options_1 = {};
            Object.keys(changes).forEach(function (prop) {
                if (prop == 'defaultDate') {
                    if (defaultDate_1) {
                        _this.calendar.gotoDate(changes.defaultDate.currentValue);
                    }
                }
                else if (prop != 'data') {
                    options_1[prop] = changes[prop].currentValue;
                }
            });
            if (Object.keys(options_1).length > 0) {
                this.calendar.setOptions(options_1);
            }
        }
    };
    SchedulerComponent.prototype.ngOnDestroy = function () {
        if (this.calendar) {
            this.calendar.destroy();
            this.calendar = null;
        }
    };
    SchedulerComponent.prototype.options = function () {
        var _this = this;
        var viewNameMap = {
            month: 'dayGridMonth',
            agendaWeek: 'timeGridWeek',
            agendaDay: 'timeGridDay'
        };
        var options = {
            plugins: [moment_1.default, daygrid_1.default, timegrid_1.default, interaction_1.default],
            defaultDate: this.defaultDate,
            locales: allLocales,
            locale: this.locale,
            editable: this.editable,
            weekends: this.weekends,
            firstDay: this.firstDayOfWeek,
            height: this.height,
            eventTimeFormat: this.timeFormat,
            defaultView: viewNameMap[this.defaultView],
            timeZone: 'UTC',
            header: {
                left: 'prev,next today',
                center: 'title',
                right: 'dayGridMonth,timeGridWeek,timeGridDay'
            }
        };
        options['eventClick'] = function (e) {
            _this.eventSelect.emit(e.event.extendedProps.data);
        };
        options['dateClick'] = function (e) { return _this.onDayClick(e); };
        options['eventResize'] = function (e) { return _this.onEventResize(e); };
        options['eventDrop'] = function (e) { return _this.onEventDrop(e); };
        options['datesRender'] = function (e) { return _this.onViewRender(e); };
        return options;
    };
    SchedulerComponent.prototype.ngAfterViewInit = function () {
        this.calendar = new core_2.Calendar(this.element.nativeElement, this.options());
        this.calendar.render();
    };
    SchedulerComponent.prototype.update = function (e) {
        var event = e.event;
        var data = event.extendedProps.data;
        if (event.start != null) {
            set(data, this.startProperty, moment_1.toMoment(event.start, this.calendar).format());
        }
        if (event.end != null) {
            set(data, this.endProperty, moment_1.toMoment(event.end, this.calendar).format());
        }
        this.eventUpdate.emit(data);
    };
    __decorate([
        core_1.Input(),
        __metadata("design:type", Object),
        __metadata("design:paramtypes", [Object])
    ], SchedulerComponent.prototype, "style", null);
    __decorate([
        core_1.Input(),
        __metadata("design:type", Array),
        __metadata("design:paramtypes", [Array])
    ], SchedulerComponent.prototype, "data", null);
    __decorate([
        core_1.Input(),
        __metadata("design:type", String)
    ], SchedulerComponent.prototype, "startProperty", void 0);
    __decorate([
        core_1.Input(),
        __metadata("design:type", String)
    ], SchedulerComponent.prototype, "idProperty", void 0);
    __decorate([
        core_1.Input(),
        __metadata("design:type", String)
    ], SchedulerComponent.prototype, "endProperty", void 0);
    __decorate([
        core_1.Input(),
        __metadata("design:type", String)
    ], SchedulerComponent.prototype, "titleProperty", void 0);
    __decorate([
        core_1.Input(),
        __metadata("design:type", String)
    ], SchedulerComponent.prototype, "defaultDate", void 0);
    __decorate([
        core_1.Input(),
        __metadata("design:type", Object)
    ], SchedulerComponent.prototype, "editable", void 0);
    __decorate([
        core_1.Input(),
        __metadata("design:type", Object)
    ], SchedulerComponent.prototype, "weekends", void 0);
    __decorate([
        core_1.Input(),
        __metadata("design:type", Object)
    ], SchedulerComponent.prototype, "defaultView", void 0);
    __decorate([
        core_1.Input(),
        __metadata("design:type", Object)
    ], SchedulerComponent.prototype, "locale", void 0);
    __decorate([
        core_1.Input(),
        __metadata("design:type", Object)
    ], SchedulerComponent.prototype, "firstDayOfWeek", void 0);
    __decorate([
        core_1.Input(),
        __metadata("design:type", String)
    ], SchedulerComponent.prototype, "timeFormat", void 0);
    __decorate([
        core_1.Output(),
        __metadata("design:type", Object)
    ], SchedulerComponent.prototype, "eventSelect", void 0);
    __decorate([
        core_1.Output(),
        __metadata("design:type", Object)
    ], SchedulerComponent.prototype, "eventUpdate", void 0);
    __decorate([
        core_1.Output(),
        __metadata("design:type", Object)
    ], SchedulerComponent.prototype, "eventAdd", void 0);
    __decorate([
        core_1.Output(),
        __metadata("design:type", Object)
    ], SchedulerComponent.prototype, "eventInit", void 0);
    __decorate([
        core_1.Output(),
        __metadata("design:type", Object)
    ], SchedulerComponent.prototype, "loadData", void 0);
    __decorate([
        core_1.ViewChild('element'),
        __metadata("design:type", core_1.ElementRef)
    ], SchedulerComponent.prototype, "element", void 0);
    SchedulerComponent = __decorate([
        core_1.Component({
            selector: 'rz-scheduler',
            template: "\n    <div #element [ngStyle]=\"style\"></div>\n  "
        })
    ], SchedulerComponent);
    return SchedulerComponent;
}());
exports.SchedulerComponent = SchedulerComponent;
var SchedulerModule = /** @class */ (function () {
    function SchedulerModule() {
    }
    SchedulerModule = __decorate([
        core_1.NgModule({
            imports: [common_1.CommonModule],
            declarations: [SchedulerComponent],
            exports: [SchedulerComponent]
        })
    ], SchedulerModule);
    return SchedulerModule;
}());
exports.SchedulerModule = SchedulerModule;
