"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var fs = require("fs-extra");
var path = require("path");
var FtpDeploy = require("ftp-deploy");
var FtpClient = require("ftp");
var url = require("url");
var path_1 = require("path");
var child_process = require("child_process");
var lodash_1 = require("lodash");
var archiver = require("archiver");
function zipFolder(srcFolder, zipFilePath, callback) {
    var output = fs.createWriteStream(zipFilePath);
    var zipArchive = archiver('zip');
    output.on('close', function () {
        callback();
    });
    zipArchive.pipe(output);
    zipArchive.bulk([{ cwd: srcFolder, src: ['**/*'], expand: true }]);
    zipArchive.finalize(function (err, bytes) {
        if (err) {
            callback(err);
        }
    });
}
function getMSDeployPath() {
    var relativeMsDeployPath = 'IIS/Microsoft Web Deploy V3/msdeploy.exe';
    var msDeploy64Path = path_1.resolve(path_1.join(process.env.ProgramFiles, relativeMsDeployPath));
    var pf32Path = process.env['ProgramFiles(x86)'];
    var msDeploy32Path = pf32Path
        ? path_1.resolve(path_1.join(pf32Path, relativeMsDeployPath))
        : null;
    if (fs.existsSync(msDeploy64Path)) {
        return msDeploy64Path;
    }
    else if (msDeploy32Path && fs.existsSync(msDeploy32Path)) {
        return msDeploy32Path;
    }
    else {
        console.log("Could not find msdeploy.exe in " + msDeploy64Path + " or " + msDeploy32Path + ". You can install it from http://www.iis.net/downloads/microsoft/web-deploy");
    }
}
function readdir(dir) {
    var _a;
    return fs.statSync(dir).isDirectory()
        ? (_a = [dir]).concat.apply(_a, fs.readdirSync(dir).map(function (f) { return readdir(path.join(dir, f)); })) : dir;
}
function deployFTP(_a) {
    var source = _a.source, server = _a.server, user = _a.user, password = _a.password, onStart = _a.onStart, onExit = _a.onExit, onError = _a.onError;
    var _b = url.parse(server), host = _b.host, port = _b.port, pathname = _b.pathname;
    try {
        var client_1 = new FtpClient();
        client_1.on('ready', function () {
            console.log("radzen: Connected to " + host);
            console.log("radzen: Deleting content of " + pathname + " ...");
            client_1.rmdir(pathname, true, function () {
                client_1.mkdir(pathname, function () {
                    console.log("radzen: Content of " + pathname + " deleted");
                    if (process.platform == 'win32') {
                        client_1.end();
                        var deploy_1 = new FtpDeploy();
                        deploy_1.on('uploading', function (data) {
                            console.log("radzen: Uploading " + data.percentComplete + "% (" + data.transferredFileCount + " of " + data.totalFileCount + "), " + data.filename);
                        });
                        deploy_1.on('error', function (data) {
                            console.log("radzen: " + data.err);
                        });
                        deploy_1.on('upload-error', function (data) {
                            console.log("radzen: " + data.err);
                        });
                        deploy_1.deploy({
                            host: host,
                            port: port,
                            username: user,
                            password: password,
                            localRoot: source,
                            remoteRoot: pathname
                        }, function (err) {
                            if (err) {
                                console.log("radzen: " + err);
                            }
                            else {
                                console.log("radzen: Upload finished.");
                                onExit();
                            }
                        });
                    }
                    else {
                        var files_1 = readdir(source);
                        var _loop_1 = function (i) {
                            var file = files_1[i];
                            var toPath = path.join(pathname, file.replace(source, ''));
                            var callback = function (err) {
                                console.log("radzen: Uploading " + Math.round((i / files_1.length) * 100) + "% (" + i + " of " + files_1.length + "), " + toPath);
                                if (err) {
                                    console.log("radzen: " + err);
                                    client_1.end();
                                    onError(err);
                                    return;
                                }
                                if (i == files_1.length - 1) {
                                    client_1.end();
                                    console.log("radzen: Upload finished.");
                                    onExit();
                                }
                            };
                            if (toPath != pathname) {
                                if (fs.statSync(file).isDirectory()) {
                                    client_1.mkdir(toPath, callback);
                                }
                                else {
                                    client_1.put(file, toPath, callback);
                                }
                            }
                        };
                        for (var i = 0; i < files_1.length; i++) {
                            _loop_1(i);
                        }
                    }
                });
            });
        });
        client_1.on('error', function (err) {
            console.log('radzen: ' + err);
            onError(err);
        });
        client_1.connect({
            host: host,
            port: port,
            user: user,
            password: password,
            secure: true
        });
    }
    catch (e) {
        console.log('radzen: ' + e);
    }
}
exports.deployFTP = deployFTP;
function deployIIS(_a) {
    var source = _a.source, provider = _a.provider, destination = _a.destination, server = _a.server, user = _a.user, password = _a.password, method = _a.method, connectionStrings = _a.connectionStrings, onStart = _a.onStart, onExit = _a.onExit, onError = _a.onError, removeAdditional = _a.removeAdditional;
    var login = provider == 'azure'
        ? "computerName=" + server + ",username=" + user + ",password=" + password + ",authType=Basic"
        : (method ||
            'computerName') + "=" + server + ",username=" + user + ",password=" + password;
    // Sync both client and server
    var syncOptions = provider == 'azure'
        ? [
            '-verb:sync',
            '-verbose=Error',
            "-source:contentpath='" + source + "'",
            "-dest:contentpath='" + destination + "'" + (server != 'localhost' ? ',' + login : '')
        ]
        : [
            '-verb:sync',
            '-verbose=Error',
            "-source:iisApp='" + source + "'",
            "-dest:iisApp='" + destination + "'" + (server != 'localhost' ? ',' + login : '')
        ].concat(method == 'wmsvc' ? ['-allowUntrusted'] : []);
    if (!removeAdditional) {
        syncOptions.push('-enableRule:DoNotDeleteRule');
    }
    console.log("radzen: Syncing deploy folder with server");
    executeMSDeployCommand({
        options: syncOptions,
        onStart: onStart,
        onExit: function () {
            console.log('radzen: Removing temporary deploy folder');
            fs.removeSync(source);
            onExit();
        },
        onError: onError
    });
}
exports.deployIIS = deployIIS;
function deploy(_a) {
    var source = _a.source, provider = _a.provider, destination = _a.destination, server = _a.server, user = _a.user, password = _a.password, method = _a.method, connectionStrings = _a.connectionStrings, localization = _a.localization, serverVersion = _a.serverVersion, application = _a.application, onStart = _a.onStart, onExit = _a.onExit, onError = _a.onError;
    var serverPath = path_1.join(source, 'server', 'bin', 'Release', 'netcoreapp3.1', 'publish');
    if (fs.existsSync(serverPath)) {
        var appSettingsFilePath = path_1.join(serverPath, 'appsettings.json');
        if (!fs.existsSync(appSettingsFilePath) &&
            fs.existsSync(path_1.join(source, 'server', 'appsettings.json'))) {
            fs.copySync(path_1.join(source, 'server', 'appsettings.json'), path_1.join(serverPath, 'appsettings.json'));
        }
        if (fs.existsSync(appSettingsFilePath)) {
            console.log('radzen: Reading connection strings');
            var connectionStringsToSave_1 = {};
            lodash_1.forEach(connectionStrings, function (connectionString, name) {
                if (connectionString.value) {
                    connectionStringsToSave_1[name + "Connection"] = connectionString.value;
                }
            });
            var appSettingsContent = fs.readFileSync(appSettingsFilePath);
            var appSettingsContentAsJSON = JSON.parse(appSettingsContent);
            lodash_1.merge(appSettingsContentAsJSON, {
                ConnectionStrings: connectionStringsToSave_1
            });
            console.log('radzen: Saving connection strings in appsettings.json');
            fs.writeFileSync(appSettingsFilePath, JSON.stringify(appSettingsContentAsJSON));
        }
        var webConfigFilePath = path_1.join(serverPath, 'web.config');
        if (!fs.existsSync(webConfigFilePath)) {
            var webConfigFileContent = fs.readFileSync(path_1.join(__dirname, './templates', 'web.config'), 'utf8');
            if (webConfigFileContent) {
                webConfigFileContent = webConfigFileContent.replace('%LAUNCHER_ARGS%', ".\\" + application.name + ".dll");
                fs.writeFileSync(webConfigFilePath, webConfigFileContent);
            }
        }
    }
    var deployPath = path_1.join(source, 'deploy');
    if (fs.existsSync(deployPath)) {
        fs.removeSync(deployPath);
    }
    var deployZip = path_1.join(source, 'deploy.zip');
    if (fs.existsSync(deployZip)) {
        fs.removeSync(deployZip);
    }
    console.log('radzen: Creating temporary deploy folder');
    fs.mkdirSync(deployPath);
    if (fs.existsSync(serverPath)) {
        console.log('radzen: Copying server app to deploy folder');
        fs.copySync(serverPath, deployPath);
    }
    if (provider == 'ftp') {
        deployFTP({
            source: deployPath,
            server: server,
            user: user,
            password: password,
            onStart: onStart,
            onExit: onExit,
            onError: onError
        });
    }
    else if (provider == 'iis') {
        deployIIS({
            source: deployPath,
            provider: provider,
            destination: destination,
            server: server,
            user: user,
            password: password,
            method: method,
            connectionStrings: connectionStrings,
            onStart: onStart,
            onExit: onExit,
            onError: onError
        });
    }
    else if (provider == 'zip') {
        zipFolder(deployPath, path_1.join(source, 'deploy.zip'), function (err) {
            if (err) {
                console.log("radzen: " + err);
                onError(err);
            }
            else {
                if (fs.existsSync(deployPath)) {
                    fs.removeSync(deployPath);
                }
                console.log("radzen: Deploy finished.");
                onExit();
            }
        });
    }
}
exports.deploy = deploy;
function executeMSDeployCommand(_a) {
    var options = _a.options, onStart = _a.onStart, onExit = _a.onExit, onError = _a.onError;
    var msDeployPath = getMSDeployPath();
    if (msDeployPath) {
        var child = child_process.spawn("\"" + msDeployPath + "\"", options, {
            shell: true
        });
        onStart(child);
        child.on('error', onError);
        child.on('exit', function (code) {
            if (code > 0) {
                onError(code);
            }
            else {
                onExit();
            }
        });
        child.stderr.on('data', function (data) {
            console.log('msdeploy:', data.toString());
        });
        child.stdout.on('data', function (data) {
            console.log('msdeploy:', data.toString());
        });
    }
}
exports.executeMSDeployCommand = executeMSDeployCommand;
//# sourceMappingURL=deploy.js.map